#include <iostream>
#include <cassert>

#include "../src/CalCommon.h"
#include "common.h"

using namespace std;

namespace {
bool isClose(double a, double b)
{
    return fabs(a)/fabs(b)-1. < 1.e-5;
}
}

int main()
{
	const unsigned BINS = 158;
	const double LOWER = -1., UPPER = 1.;
	const double BW = (UPPER-LOWER)/BINS;

	//	test binned axis
	UniformilyBinnedAxis a1(LOWER, UPPER, BINS),
						 a2(LOWER, BINS, BW);

	assert(a1.bins() == a2.bins() && a1.bins() == BINS);
	assert(isClose(a1.bin().width(), BW) && isClose(a2.bin().width(), BW));
	assert(isClose(a1.bin(BINS-1).mid(), UPPER-.5*BW)
			&& isClose(a2.bin(BINS-1).mid(), UPPER-.5*BW));
	assert(isClose(a1.lower(), LOWER) && isClose(a1.upper(), UPPER));

	//	put one in each bin
	for (unsigned i=0; i<BINS; ++i) {
		a1[i]++;
		a2[LOWER+i*BW+.5*BW]++;
	}

	for (unsigned i=0; i<BINS; ++i)
		assert(a1[i] == 1 && a2[i] == 1);


	//	NonUniformily Binned Axis
	CalReal64Vector v(0, BINS+1);
	for (unsigned i=0; i<BINS+1; ++i)
		v[i] = i*i;
	NonUniformilyBinnedAxis a3(v);

	assert(a3.bins() == BINS);
	double bw = DBL(BINS*BINS)-DBL((BINS-1)*(BINS-1));
	assert(isClose(a3.bin().width(), 1.) &&
		   isClose(a3.bin(BINS-1).width(), bw));
	assert(isClose(a3.bin().mid(), .5) &&
		   isClose(a3.bin(BINS-1).mid(), a3.upper()-.5*bw));

	//	put one in each bin
	for (unsigned i=0; i<BINS*BINS; ++i)
		a3[DBL(i)]++;

	unsigned sum = 0;
	for (unsigned i=0; i<BINS; ++i) {
		cout<<i<<" "<<a3[i]<<endl;
		assert(a3[i] == 2*i+1);
		sum += a3[i];
	}
	assert(sum == BINS*BINS);

	return 0;
}
