//
//	test the cal rawx2mm calls
//
#include <fstream>
#include <iostream>
#include <iomanip>
#include "CalServer.h"
#include "Xmm.h"
#include "GeometryDataServer.h"
#include "common.h"

using namespace std;

int main()
{
	for (unsigned i = 0; i < xmmPayLoad.instruments().size(); ++i) {

		XmmInstrument *ins = xmmPayLoad.instruments()[i];
		if (ins->id() == Xmm::ERM)
			continue;

		//	open xmgr parameter file
		strstream s;
		s<<"ccds_"<<lowercase(Xmm::toString(ins->id()))<<".dat"<<ends;
		ofstream set(s.str());
		set<<"@title \"Positions of CCDs in "<<Xmm::toString(ins->id())
		   <<"\"\n";
		set<<"@xaxis label \"x [mm]\"\n";
		set<<"@yaxis label \"y [mm]\"\n";
		set<<"@string loctype world\n";
		set<<"@string just 2\n";

		cerr<<"Instrument: "<<Xmm::toString(ins->id())<<endl;
		calServer.setInstrument(ins->id());
		calServer.state()->randomize()->set(false);
		if (calServer.state()->node()->isActive())
			calServer.state()->node()->set(Xmm::PRIMARY_NODE);
		if (calServer.state()->ocb()->isActive())
			calServer.state()->ocb()->set(1);
		calServer.state()->temperature()->set(293.);
		calServer.state()->temperatureCamera()->set(293.);

		for (unsigned ccd = 1; ccd<=ins->nCcds(); ++ccd) {
			calServer.state()->ccd()->set(ccd);

			short minX = ins->minX(),
				  maxX = ins->maxX(),
				  minY = ins->minY(),
				  maxY = ins->maxY();
			short rawX[] = { minX, maxX, maxX, minX, minX },
				  rawY[] = { minY, minY, maxY, maxY, minY };
			CalInt16Vector rawXVec(rawX, NUM_OF(rawX)),
						   rawYVec(rawY, NUM_OF(rawY));
			double xmm[NUM_OF(rawX)], ymm[NUM_OF(rawX)], zmm[NUM_OF(rawX)];

			GeometryDataServer *cdf = calServer.getAtom(cdf);
			XmmPixelList pl = cdf->chipCoordList(rawXVec, rawYVec);
			XmmCartesianPointList cpl = pl.toCamCoord1(xmm, ymm, zmm);
			cpl.toCamCoord2();

			cerr<<"CCD "<<ccd<<endl;
			for (unsigned n=0; n<NUM_OF(rawX); ++n) {
				cerr<<rawXVec[n]<<"/"<<rawYVec[n]<<" = ";
				cerr<<setprecision(10)<<xmm[n]<<"/"<<ymm[n]<<"/"<<zmm[n]<<endl;
			}

			double cenx = .25 * (xmm[0]+xmm[1]+xmm[2]+xmm[3]),
				   ceny = .25 * (ymm[0]+ymm[1]+ymm[2]+ymm[3]);
			//
			//	write CCD number
			//
			set<<"@with string\n";
			set<<"@string on\n";
			set<<"@string char size 2\n";
			set<<"@string "<<cenx<<", "<<ceny<<endl;
			set<<"@string def \""<<ccd<<"\"\n";

			//
			//	coordinate systems: x
			//
			set<<"@with line\n";
			set<<"@line on\n";
			set<<"@line loctype world\n";
			set<<"@line "<<xmm[0]<<", "<<ymm[0]<<", "
						 <<xmm[1]<<", "<<ymm[1]<<endl;
			set<<"@line arrow 2\n";
			set<<"@line arrow size 1.5\n";
			set<<"@line def\n";
			set<<"@with string\n";
			set<<"@string on\n";
			set<<"@string char size 1\n";
			set<<"@string "<<xmm[1]<<", "<<ymm[1]<<endl;
			set<<"@string def \"X\"\n";

			//
			//	coordinate systems: y
			//
			set<<"@with line\n";
			set<<"@line on\n";
			set<<"@line loctype world\n";
			set<<"@line "<<xmm[0]<<", "<<ymm[0]<<", "
						 <<xmm[3]<<", "<<ymm[3]<<endl;
			set<<"@line arrow 2\n";
			set<<"@line arrow size 1.5\n";
			set<<"@line def\n";
			set<<"@with string\n";
			set<<"@string on\n";
			set<<"@string char size 1\n";
			set<<"@string "<<xmm[3]<<", "<<ymm[3]<<endl;
			set<<"@string def \"Y\"\n";

			//
			//	write data
			//
			set<<"@target s"<<ccd-1<<endl;
			set<<"@s"<<ccd-1<<" color 1\n";
			set<<"@type xy\n";

			for (unsigned n=0; n<NUM_OF(rawX); ++n)
				set<<xmm[n]<<" "<<ymm[n]<<endl;

			set<<"&\n";

		}
		set.close();
	}

	return 0;
}
