//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	CalAtomsMisc.h --- miscellaneous Cal atoms
//
//	DESCRIPTION:
//
//	CHANGE HISTORY:
//		UL, 1997-10-25: 1.0.0: - skeleton
//		UL, 1997-??-??: 1.1.0: - extended
//		UL, 1998-08-28: 1.2.0: - EnergyCorrector class added
//		UL, 2000-04-25: 1.3.0: - additional mode parameter data
//
//	AUTHOR:
//		U. Lammers, Mon Oct  6 15:32:57 WET DST 1997
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/cal/src/CalAtomsMisc.h,v 1.2 2001/10/29 16:57:55 ulammers Exp $
//

#ifndef CAL_ATOMS_MISC_H
#define CAL_ATOMS_MISC_H

#include "Ccf.h"
#include "CalAtomsBase.h"
#include "Celestial.h"

#include <vector>

//
//	---( background )------------------------------------------------------
//
class Background;

class BackgroundSpectrum : public std::vector<double> {
public:
	BackgroundSpectrum() : _nChan(0) { PROC ENDPROC }

	void pushChannel(double value) { PROC
									 _channs.push_back(_nChan++);
									 this->push_back(value);
									 ENDPROC }
	const std::vector<unsigned> &channels() const { return _channs; }

	~BackgroundSpectrum() { PROC ENDPROC }

private:
	std::vector<unsigned> _channs;
	unsigned			  _nChan;
};


class Background : public CalAtom {
public:
	friend class CalAtomCollection<Background>;

	//	default constructor - does nothing
	Background() { PROC ENDPROC }

	static unsigned id() { PROC RETURN Ccf::Background; }

	void dump() const;

	BackgroundSpectrum particle(double time = 0, double r = 0.) const;
	BackgroundSpectrum diffuseX(double time = 0, double r = 0.) const;

private:
	double				_particleP0, _particleP1;
	BackgroundSpectrum	_diffuse;
	double				_diffModP0, _diffModP1, _diffModP2;
}; // Background


//
//	---( mode parameters )------------------------------------------------
//
class ModeParameters : public CalAtom {
public:
	friend class CalAtomCollection<ModeParameters>;

	//	default constructor - does nothing
	ModeParameters() { PROC ENDPROC }

	static unsigned id() { PROC RETURN Ccf::ModeParam; }

	float integrationTime() const { PROC RETURN _integrationTime; }
	float shiftTime() const { PROC RETURN _shiftTime; }
	float frameTime() const { PROC RETURN _frameTime; }
	float cycleTime() const { PROC RETURN _cycleTime; }
	float timeResolution() const { PROC RETURN _timeResolution; }
	float dutyCycle() const { PROC RETURN _dutyCycle; }
	float smearedFraction() const { PROC RETURN _smearedFraction; }
	float pointSourcePileUp() const { PROC RETURN _pointSourcePileUp; }
	float limitingFlux() const { PROC RETURN _limitingFlux; }
	short underX() const { PROC RETURN _underX; }
	short overX() const { PROC RETURN _overX; }
	short underY() const { PROC RETURN _underY; }
	short overY() const { PROC RETURN _overY; }
	short windowX0() const { PROC RETURN _windowX0; }
	short windowY0() const { PROC RETURN _windowY0; }
	short windowDX() const { PROC RETURN _windowDX; }
	short windowDY() const { PROC RETURN _windowDY; }

	~ModeParameters() { PROC ENDPROC }

private:
	float _integrationTime,
		  _shiftTime,
		  _frameTime,
		  _cycleTime,
		  _timeResolution,
		  _dutyCycle,
		  _smearedFraction,
		  _pointSourcePileUp,
		  _limitingFlux;
	short _underX,
		  _overX,
		  _underY,
		  _overY,
		  _windowX0,
		  _windowY0,
		  _windowDX,
		  _windowDY;
};


//
//	--- ( HK windows )-----------------------------------------------------
//
class HkWindows : public CalAtom {
public:
	friend class CalAtomCollection<HkWindows>;

	//	default constructor - does nothing
	HkWindows() { PROC ENDPROC }

	void dump() const;

	~HkWindows() { PROC ENDPROC }

private:
	//	data members
};

#endif

